const productsModel = require('./productsModel');
const pricesModel = require('./pricesModel');
const notificationsService = require('../notifications/notificationsService');

async function createProductsWithPrices(products) {
    if (!Array.isArray(products) || products.length === 0) {
        throw new Error('Invalid products data. Expected a non-empty array.');
    }

    try {
        const productUrls = products.map(product => product.url);
        const existingProducts = await productsModel.getProductsByUrls(productUrls);

        const existingProductUrls = existingProducts.map(product => product.url);
        const newProducts = products.filter(product => !existingProductUrls.includes(product.url));

        let createdProducts = [];
        if (newProducts.length > 0) {
            const newProductsData = newProducts.map(product => ({ url: product.url }));
            createdProducts = await productsModel.createProducts(newProductsData);
        }

        const allProducts = [...existingProducts, ...createdProducts];
        const productsPrices = products.map(product => {
            const existingProduct = allProducts.find(p => p.url === product.url);
            if (!existingProduct) {
                throw new Error(`Product not found for URL: ${product.url}`);
            }
            return {
                productId: existingProduct.id,
                price: product.price,
            };
        });

        pricesModel.createPrices(productsPrices).catch(error => {
            console.error('Error creating prices:', error);
        });

        checkPriceDropNotifications(allProducts.map(product => ({
            id: product.id,
            url: product.url,
            price: products.find(p => p.url === product.url).price,
        }))).catch(error => {
            console.error('Error checking price drop notifications:', error);
        });
    } catch (error) {
        console.error('Error creating products with prices:', error);
        throw error;
    }
}

async function checkPriceDropNotifications(products) {
    try {
        const productIds = products.map(product => product.id);
        const lastPrices = await pricesModel.getLastPricesByProductIds(productIds);

        products.forEach(product => {
            const lastPrice = lastPrices.find(price => price.productId === product.id);
            if (lastPrice && product.price < lastPrice.price * 0.4) {
                notificationsService.sendPriceDropNotification(product.url, lastPrice.price, product.price);
            }
        });
    } catch (error) {
        console.error('Error checking price drop notifications:', error);
        throw error;
    }
}

module.exports = {
    createProductsWithPrices,
};